import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_essentials/state/state_observer.dart';
import 'package:sunny_lifecycle/sunny_lifecycle.dart';

AppContextEvents get appContextEvents => sunny.get();

typedef AppContextHandler = FutureOr Function(BuildContext context);

/// It's common that you want to fire an event, but only within the "app" context, ie.
/// inside of a [MaterialApp] or [CupertinoApp].  This service lets you fire those
/// events anywhere in the codebase, and they will get picked up and run inside
/// the appropriate context.
class AppContextEvents implements LifecycleAware {
  static AppContextEvents of(BuildContext context) {
    return sunny.get(context: context);
  }

  final _buffer = <AppContextHandler>[];
  final _events = StreamController<AppContextHandler>.broadcast();
  bool _isListening = false;
  final uid = uuid();

  AppContextEvents() {
    _events.onListen = () {
      this._isListening = true;
      for (var event in _buffer) {
        _events.add(event);
      }
    };
  }

  /// Runs the block of code with a BuildContext that resides inside the "App",
  /// [CupertinoApp] or [MaterialApp].  This code will only run once.
  void doWithAppContext(AppContextHandler onTap) {
    if (!_isListening) {
      _buffer.add(onTap);
    } else {
      _events.add(onTap);
    }
  }

  /// Listens to the events coming from the stream.  This should only happen
  /// once for the lifecycle of your code.
  Stream<AppContextHandler> get stream => _events.stream;

  @override
  FutureOr dispose() {
    if (!_events.isClosed) {
      _events.close();
    }
  }
}

class NavigatorContextWidget extends StatefulWidget {
  final Widget? child;

  const NavigatorContextWidget({Key? key, this.child}) : super(key: key);

  @override
  _NavigatorContextWidgetState createState() => _NavigatorContextWidgetState();
}

class _NavigatorContextWidgetState extends BaseState<NavigatorContextWidget> {
  StreamSubscription? _sub;
  late BuildContext _lastContext;

  @override
  Widget build(BuildContext context) {
    _lastContext = context;
    _sub ??= AppContextEvents.of(context).stream.listen((event) {
      event(_lastContext);
    });
    return widget.child!;
  }

  @override
  void dispose() {
    _sub?.cancel();
    super.dispose();
  }
}
