import 'package:firebase_auth/firebase_auth.dart' as fb;
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:logging/logging.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/login/apple_login_provider.dart';
import 'package:sunny_lifecycle/login/google_login_provider.dart';

import 'email_password_login_provider.dart';

// final GoogleSignIn googleSignIn = GoogleSignIn();
final fb.FirebaseAuth _auth = fb.FirebaseAuth.instance;
final _log = Logger("loginHandler");

abstract class LoginProvider {
  static const google = GoogleLoginProvider();
  static const apple = AppleLoginProvider();
  // static const facebook = FacebookLoginProvider();
  static const email = EmailPasswordLoginProvider();

  bool get isEnabled;

  String get name;

  String get title => name.capitalize();

  @protected
  Future<AuthResponse> doAttemptLogin(BuildContext context);

  @protected
  Future<AuthResponse> doAttemptRegister(BuildContext context) {
    return doAttemptLogin(context);
  }

  Future<AuthResponse> login(BuildContext context) async {
    return _doAuth(context, doAttemptLogin);
  }

  Future<AuthResponse> register(BuildContext context) async {
    return _doAuth(context, doAttemptRegister);
  }

  Future<AuthResponse> _doAuth(BuildContext context,
      Future<AuthResponse> login(BuildContext context)) async {
    try {
      final response = await login(context)
          .timeout(1.minute, onTimeout: () => AuthResponse.cancelled());
      if (response.cred != null) {
        _log.info("Continuing with login...");
        final fb.User? user =
            (await _auth.signInWithCredential(response.cred!)).user;
        return response.withUser(user);
      }
      return response;
    } catch (e, stack) {
      if (e is PlatformException) {
        switch (e.code) {
          case 'ERROR_ACCOUNT_EXISTS_WITH_DIFFERENT_CREDENTIAL':
            return AuthResponse.error(
                "The email address you are attempting to use has already been setup.  Please log in",
                AuthErrorCode.duplicateAccount);
          case 'user-disabled':
            return AuthResponse.error(
                "This account has been disabled", AuthErrorCode.disabled);
          default:
            return AuthResponse.error("There was an error logging you in");
        }
      } else if (e is fb.FirebaseAuthException) {
        switch (e.code) {
          case 'user-disabled':
            return AuthResponse.error(
                "This account has been disabled", AuthErrorCode.disabled);
          default:
            return AuthResponse.error(e.message);
        }
      } else {
        _log.severe("Authentication issue: $e", e, stack);
        return AuthResponse.error("There was an error logging you in");
      }
    }
  }

  Future signOut();

  Widget logo(double size, {Color color = Colors.white}) {
    return CircleAvatar(
        radius: size / 2,
        backgroundColor: color,
        child: Image.asset("assets/images/$name.png",
            height: size, width: (24.0 / 32.0) * size));
  }

  const LoginProvider();
}

enum AuthErrorCode { disabled, duplicateAccount, invalidCredentials, other }

class AuthResponse {
  final fb.AuthCredential? cred;
  final Object? error;
  final AuthErrorCode? code;
  final fb.User? user;
  final bool reauth;

  const AuthResponse.error(this.error, [this.code = AuthErrorCode.other])
      : user = null,
        reauth = false,
        cred = null;

  const AuthResponse.success(this.cred, {this.user, this.reauth = true})
      : error = null,
        code = null;

  const AuthResponse.userLogin({
    this.user,
  })  : error = null,
        cred = null,
        reauth = false,
        code = null;

  const AuthResponse.cancelled()
      : cred = null,
        reauth = false,
        user = null,
        code = null,
        error = null;

  bool get isCancelled => cred == null && error == null;

  bool get isLoggedIn => user != null;

  AuthResponse withUser(fb.User? user) {
    assert(this.cred != null);
    return AuthResponse.success(this.cred, user: user, reauth: false);
  }

  LoginState get status {
    return user != null
        ? LoginState.loggedIn
        : error != null
            ? LoginState.error
            : LoginState.cancelled;
  }
}

enum LoginState {
  loggingIn,
  incomplete,
  loggedIn,
  loadingProfile,
  cancelled,
  error
}

extension LoginStateExt on LoginState {
  bool get isWorking {
    return this == LoginState.loggingIn || this == LoginState.loadingProfile;
  }
}
