import 'dart:async';

import 'package:dartx/dartx.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:logging/logging.dart';
import 'package:sunny_dart/helpers/strings.dart';
import 'package:sunny_dart/typedefs.dart';
import 'package:sunny_lifecycle/analytics/analytics_service.dart';
import 'package:sunny_lifecycle/app_info.dart';
import 'package:sunny_lifecycle/auth_state.dart';
import 'package:sunny_lifecycle/core_info.dart';
import 'package:sunny_lifecycle/init_interface.dart'
    if (dart.library.io) 'package:sunny_lifecycle/init_native.dart'
    if (dart.library.js) 'package:sunny_lifecycle/init_web.dart';

import 'package:sunny_lifecycle/lifecycle_main_default_resolver.dart';
import 'package:sunny_lifecycle/lifecycle_main_defaults.dart';
import 'package:sunny_lifecycle/login/login_provider.dart';
import 'package:sunny_lifecycle/push_notification_service.dart';
import 'package:sunny_lifecycle/routing2/routing_system.dart';
import 'package:sunny_lifecycle/user_preferences_service.dart';
import 'package:sunny_sdk_core/api/api_exceptions.dart';
import 'package:sunny_sdk_core/auth/auth_user_profile.dart';
import 'package:sunny_sdk_core/services.dart';

abstract class AuthModule<U extends Object> {
  Future<AuthUserProfile?> getInitialUser();

  Iterable<LoginProvider> get loginProviders;

  Future<UserDetails> loadProfile(U user);

  IAuthState initializeState(
      AuthUserProfile? initialProfile, Stream<ErrorStack> appErrors);

  FutureOr signOut(BuildContext context);
}

abstract class InjectionModule {
  FutureOr<BuildContextResolver> initializeResolver();

  BuildContextResolver get serviceResolver;
}

abstract class PushNotificationModule {
  PushNotificationService initializePushNotifications(
      Getter<BuildContext?> context, AppInfo appInfo);
}

class DefaultPushNotificationModule
    implements PushNotificationModule, RegisteringModule {
  final Future<bool> Function(BuildContext context, Object user, String token)
      sendDevicePushToken;
  final dynamic Function(String? type, String? body) pushMessageBodyConverter;

  const DefaultPushNotificationModule({
    required this.sendDevicePushToken,
    this.pushMessageBodyConverter = defaultPushMessageBodyConverter,
  });

  @override
  List<Inst> registrations() => [
        if (!kIsWeb) Inst.instance(FirebaseMessaging.instance),
      ];

  @override
  PushNotificationService initializePushNotifications(
      Getter<BuildContext?> context, AppInfo appInfo) {
    final pushService = PushNotificationService(
      context,
      appInfo,
      userStateStream: globalUserStateChanged,
      convertMessage: this.pushMessageBodyConverter,
      sendDevicePushToken: this.sendDevicePushToken,
    );

    // final res = sunny.get<BuildContextResolver>(context: context);
    // res.registerSingleton(context, _push);
    return pushService;
  }
}

abstract class LocalizationModule {
  Future<InfoX?> initializeInfoX();

  Future<SunnyLocalization?> initializeLocalization(InfoX? infoX);
}

class DefaultLocalizationModule implements LocalizationModule {
  @override
  Future<SunnyLocalization?> initializeLocalization(InfoX? infoX) async {
    return await infoX?.initializeLocalization();
  }

  @override
  Future<InfoX?> initializeInfoX() async {
    return SunnyInfoX();
  }

  const DefaultLocalizationModule();
}

abstract class UserPrefsModule {
  UserPreferencesService initializePreferences(String identifier);
}

class DefaultUserPrefsModule implements UserPrefsModule {
  @override
  UserPreferencesService initializePreferences(String identifier) {
    return UserPreferencesService(identifier);
  }

  const DefaultUserPrefsModule();
}

class ProviderInjectionModule implements InjectionModule {
  const ProviderInjectionModule();

  @override
  FutureOr<BuildContextResolver> initializeResolver() {
    final resolver = ProviderBuildContextResolver();
    if (sunny is! SunnyCore) {
      sunny = SunnyCore(resolver: resolver);
    } else {
      sunny.core.resolver = resolver;
    }
    return resolver;
  }

  @override
  BuildContextResolver get serviceResolver {
    return sunny.core.resolver!;
  }
}

abstract class RegisteringModule {
  List<Inst> registrations();
}

abstract class AnalyticsModule {
  AnalyticsService initializeAnalytics();
  List<AnalyticsProvider>? get providers;
}

class DefaultAnalyticsModule implements AnalyticsModule {
  @override
  final List<AnalyticsProvider>? providers;

  const DefaultAnalyticsModule([this.providers]);

  @override
  AnalyticsService initializeAnalytics() {
    if (providers != null) {
      defaultAnalyticsProviders = providers!;
    }
    return analytics;
  }
}

///
/// Responsible for setting up navigation elements
///
abstract class RoutingModule {
  Future initializeRoutes();

  RoutingSystem createRoutingSystem([RouterDelegate? parent]);
}

///
/// Responsible for retrieving device info from the native device or browser.
///
abstract class DeviceInfoModule {
  Future<DeviceInfo> loadDeviceInfo();
}

class DefaultDeviceInfoModule implements DeviceInfoModule {
  const DefaultDeviceInfoModule();

  static final log = Logger('deviceInfoModule');

  @override
  Future<DeviceInfo> loadDeviceInfo() async {
    DeviceInfo _deviceInfo;
    try {
      _deviceInfo = await infoX.loadDeviceInfo().timeout(1.seconds);
    } catch (e) {
      log.info("Problem getting device info: $e");
      _deviceInfo = DeviceInfo(
        geo: null,
        isSimulator: false,
        softwareVersion: "unknown",
        locale: "en-US",
        language: "en",
        deviceType: "unknown",
        deviceModel: "unknown",
        deviceBrand: "unknown",
        ipAddress: "unknown",
        deviceId: uuid(),
        software: "unknown",
      );
    }
    return _deviceInfo;
  }
}

abstract class AppInfoModule {
  Future<AppInfo> loadAppInfo(String identifier, String appTitle);
}

class DefaultAppInfoModule implements AppInfoModule {
  final FallbackAppInfo? fallbackAppInfo;

  const DefaultAppInfoModule([this.fallbackAppInfo]);

  @override
  Future<AppInfo> loadAppInfo(String identifier, String appTitle) async {
    final appInfo = await onAppInit(
        Level.INFO,
        fallbackAppInfo ??
            FallbackAppInfo(
                packageName: identifier,
                appName: appTitle,
                appleAppID: 'unknown',
                appleAppName: 'unknown',
                appBaseUrl: null));
    return appInfo;
  }
}
