import 'dart:async';

import 'package:flutter/widgets.dart' hide Notification;
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/push_notification_service.dart';

typedef MessageConverter = dynamic Function(String? envelopeType, String? body);

abstract class PushPlatform {
  Future cancel();
  Future<String?> getToken({bool force = false});
  Future requestPermissions();
  Future subscribe(BuildContext context, {bool configureBackground = false});
  Future initialize(MessageConverter messageConverter);
  Stream<SourcedPushMessage> get messages;
  Stream<String?> get pushTokens;
}

mixin PushPlatformMixin implements PushPlatform, HasDisposers {
  String? currToken;
  final _pushTokens = StreamController<String?>.broadcast();

  MessageConverter? convertMessage;
  bool newPushToken(String? token) {
    if (token != currToken) {
      currToken = token;
      _pushTokens.add(token);
      return true;
    } else {
      return false;
    }
  }

  /// Loads push message from fcm payload.
  PushMessageContent extractMessage(Map<String, dynamic> input) {
    final type = input["envelopeType"] as String?;
    dynamic body;
    final payload = input["contentPayload"] as String?;
    if (payload != null) {
      body = convertMessage?.call(type, input["contentPayload"] as String?);
    }

    final Notification? notification = input["isNotification"] != "true"
        ? null
        : (Notification()
          ..title = input["title"] as String?
          ..subtitle = input["subtitle"] as String?
          ..body = input["body"] as String?
          ..imageUrl = (input["imageUrl"] as String?)?.toUri());
    return PushMessageContent()
      ..envelopeType = type
      ..body = body
      ..nativePush = false
      ..notification = notification;
  }

  void newMessage(PushSource source, PushMessageContent content) {
    _messages.add(SourcedPushMessage(content, source));
  }

  void newRawMessage(PushSource source, Map<String, dynamic> content) {
    _messages.add(SourcedPushMessage(extractMessage(content), source));
  }

  Future<String?> internalGetToken();

  @override
  Future<String?> getToken({bool force = false}) async {
    if (force || currToken == null) {
      await requestPermissions();
      final token = await internalGetToken();
      newPushToken(token);
      return token;
    }
    return currToken;
  }

  final _messages = StreamController<SourcedPushMessage>.broadcast();
  Stream<SourcedPushMessage> get messages => _messages.stream;

  @override
  Stream<String?> get pushTokens => _pushTokens.stream;

  Future closePushStreams() async {
    await _pushTokens.close();
    await _messages.close();
  }
}
