import 'dart:collection';

import 'package:equatable/equatable.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:flutter/widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';

/// Represents the display parameters of a tab
abstract class TabDisplay {
  String? get tabLabel;

  IconData get tabIcon;
}

/// A set of tabs, including the default tab.
class TabSet with IterableMixin<TabState> {
  final int defaultIndex;
  final List<TabState> tabs;

  TabSet({this.defaultIndex = 0, required this.tabs})
      : assert(!tabs.any((tab) => tab.tabIndex == null),
            "No tabs index can be null"),
        assert(tabs.length >= defaultIndex, "defaultIndex must be in tabSet"),
        assert(tabs.map((t) => t.tabIndex).toSet().length == tabs.length,
            "Must have unique tabs");

  const TabSet.empty()
      : tabs = const [],
        defaultIndex = -1;

  factory TabSet.indexed({int defaultIndex = 0, required List<TabState> tabs}) {
    return TabSet(
        defaultIndex: defaultIndex,
        tabs: tabs.mapIndexed((item, index) => item.withIndex(index)).toList());
  }

  TabState operator [](int? index) {
    index ??= defaultIndex;
    return tabs[index];
  }

  @override
  Iterator<TabState> get iterator => tabs.iterator;
}

/// The state of a single tab.
class TabState extends Equatable implements TabDisplay {
  /// A navigator key that can be used to retrieve the navigation stack for this tab
  final GlobalKey<NavigatorState> navigatorKey;

  /// The default route for this tab
  final AppRoute route;

  /// A scroll controller, used to do scrolling when navigating to a tab
  final ScrollController scroller;

  /// The label for this tab
  @override
  final String? tabLabel;

  /// THe icon to be displayed
  @override
  final IconData tabIcon;

  ///Which index this tab is placed at
  final int? tabIndex;

  final bool isDesktop;

  TabState({
    required this.tabLabel,
    required this.tabIcon,
    required this.route,
    this.isDesktop = true,
    required int this.tabIndex,
    required this.navigatorKey,
  }) : scroller = ScrollController(debugLabel: "Tab ${route.route}");

  TabState.floating({
    required this.tabLabel,
    required this.tabIcon,
    required this.route,
    this.isDesktop = true,
    required this.navigatorKey,
  })   : tabIndex = null,
        scroller = ScrollController(debugLabel: "Tab ${route.route}");

  factory TabState.ofAppRouteMatch(AppRouteMatch match,
      {required IconData tabIcon,
      required int tabIndex,
        bool isDesktop = true,
      required GlobalKey<NavigatorState> key}) {
    return TabState(
      route: match.route!,
      tabLabel: match.route!.routeTitle(),
      tabIcon: tabIcon,
      isDesktop: isDesktop,
      navigatorKey: key,
      tabIndex: tabIndex,
    );
  }

  void dispose() {
    scroller.dispose();
  }

  @override
  List<Object?> get props {
    return [navigatorKey, tabLabel, tabIcon];
  }

  TabState withIndex(int index) {
    return TabState(
        tabLabel: tabLabel,
        tabIcon: tabIcon,
        route: route,
        tabIndex: index,
        navigatorKey: navigatorKey);
  }
}
