import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:logging_config/logging_config.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:sunny_lifecycle/lifecycle_main.dart';
import 'package:sunny_lifecycle/modules/testing_modules.dart';
import 'package:sunny_lifecycle/routing/route_history.dart';
import 'package:sunny_lifecycle/routing2/fluro_routing_system.dart';
import 'package:sunny_lifecycle/routing2/frouter_portal.dart';

import 'mock_firebase.dart';
import 'test_routing_widgets.dart';

final globalNavigatorKey = GlobalKey<NavigatorState>(debugLabel: 'global');
final tabNavigatorKey = GlobalKey<NavigatorState>(debugLabel: 'tabs');
final page1NavigatorKey = GlobalKey<NavigatorState>(debugLabel: 'page1');

void main() {
  group("Nested Routes", () {
    // TestWidgetsFlutterBinding.ensureInitialized(); Gets called in setupFirebaseAuthMocks()
    setupFirebaseAuthMocks();

    setUpAll(() async {
      await Firebase.initializeApp();
    });

    testWidgets("Nested route change", (WidgetTester tester) async {
      var history = TestCaptureHistoryObserver();

      try {
        var testAppWidget = await buildTestApp(initialPage: '/tabs', historyObserver: history);
        await tester.pumpWidget(testAppWidget);
        await tester.pageIsFound();
        await tester.changeTab(1);
        await tester.openRecordPage('123456');
        await tester.openRecordPage('654321');

        await tester.changeTab(3);

        await tester.openRecordPage('99999');
        await tester.changeTab(2);

        // Now change
        expect(
            history.events.first.snapshot.map((r) => r.route),
            containsAllInOrder([
              '/page3',
              '/page1/654321',
              '/page1/123456',
              '/page1',
              '/page2',
            ]));
      } catch (e) {
        print('----------------------------');
        print(history.events.map((e) => e.item).join("\n"));
        print('----------------------------');
        rethrow;
      }
    });
  });
}

Future<Widget> buildTestApp({String initialPage = '/', required HistoryObserver historyObserver}) async {
  configureLogging(LogConfig(handler: LoggingHandler.dev()));

  // var firebase = await Firebase.initializeApp();

  final firebase = await Firebase.initializeApp();

  return await (AppBootstrap(
    identifier: 'TEST',
    firebaseApp: firebase,
    appTitle: 'TEST',
    appInfo: TestAppInfoModule(),
    deviceInfo: TestDeviceInfoModule(),
    userPrefs: TestUserPrefsModule(),
    localization: TestLocalizationModule(),
    routing: FluroRoutingModule(
      path: initialPage,
      navigatorKey: globalNavigatorKey,
      preferUpdates: true,
      historyObservers: [historyObserver],
      initialRoute: initialPage,
      routes: buildMainRouter(historyObserver),
    ),
  ).buildRootWidget());
}

FRouter buildMainRouter(HistoryObserver observer) {
  final FRouter routes = newTestRouter();

  routes.page(
      "/tabs",
      (context, parameters) => Builder(
            builder: (context) => Scaffold(
              bottomNavigationBar: RouteTestTabs(
                pageCount: 3,
              ),
              body: RoutingPortal(
                routing: FluroRoutingModule(
                  path: '/tabs',
                  routes: buildTabRouter(observer),
                  preferUpdates: true,
                  navigatorKey: tabNavigatorKey,
                  initialRoute: '/page2',
                  historyObservers: [observer],
                ),
              ),
            ),
          ));

  return routes;
}

FRouter buildTabRouter(HistoryObserver observer) {
  final FRouter tabRoutes = newTestRouter();

  tabRoutes.page(
      "/page1",
      (context, parameters) => RoutingPortal(
            routing: FluroRoutingModule(
              routes: buildPage1Routes(),
              navigatorKey: page1NavigatorKey,
              initialRoute: '/page1/home',
              historyObservers: [observer],
              path: '/page1',
            ),
          ));
  tabRoutes.page("/page2", (context, parameters) => RouteTestPage(title: 'Page 2'));
  tabRoutes.page("/page3", (context, parameters) => RouteTestPage(title: 'Page 3'));

  return tabRoutes;
}

FRouter buildPage1Routes() {
  final FRouter innerRoutes = newTestRouter();

  innerRoutes.page<dynamic, RouteParams>(
    "/page1/home",
    (context, params) => RouteTestPage(title: 'Page 1'),
  );
  innerRoutes.page<dynamic, RouteParams>("/page1/{recordId}", (context, parameters) {
    final recordId = parameters['recordId'] as String?;
    return RouteTestPage(title: 'Record ${recordId}');
  });

  return innerRoutes;
}

typedef WidgetListBuilder = List<Widget> Function(BuildContext context);
