import 'dart:convert';
import 'dart:io';

import 'package:http/io_client.dart';
import 'package:recase/recase.dart';

/// A single icon definition read from the google doc
class IconDefinition {
  final String? iconName;
  final String? iconVariableName;
  final Uri? iconOutlineUri;
  final Uri? iconSolidUri;

  IconDefinition(
      {this.iconName,
      this.iconVariableName,
      this.iconOutlineUri,
      this.iconSolidUri});

  String get outlineUriId => iconOutlineUri!.pathSegments[1];

  String get solidUriId => iconSolidUri!.pathSegments[1];
}

const docId = "1C848ivOI_pTy7vZuaCO1KYuLH_bYhAAWo6I9dJCPtG0";

/// This is the URL to the sunny icon spreadsheet in google docs (a read-only url)
const sheetUrl =
    "https://docs.google.com/spreadsheets/u/1/d/$docId/export?format=csv&id=$docId&gid=0";

Future<Iterable<IconDefinition>> readIconDefinitions([IOClient? client]) async {
  client ??= IOClient(HttpClient()..findProxy = null);
  print("Fetching latest google doc");
  final body = await client.get(Uri.parse(sheetUrl)).then((resp) {
    if (resp.statusCode != 200) {
      throw "Failed request: ${resp.statusCode}: ${resp.body}";
    }
    return resp.body;
  });

  print("Done fetching doc");

  return body
      .split("\n")
      .skip(1)
      .map((l) => l.trim())
      .where((l) => l.isNotEmpty == true)
      .map((line) => line
          .split(",")
          .map((_) => _.trim())
          .where((_) => _.isNotEmpty == true)
          .toList())
      .map((parts) {
    final iconName = parts[0];
    final toSnakeCase = ReCase(iconName).snakeCase;

    return IconDefinition(
      iconName: toSnakeCase,
      iconVariableName: iconName,
      iconOutlineUri: Uri.parse(parts[1]),
      iconSolidUri: Uri.parse(parts[2]),
    );
  });
}

class ServerApiKey {
  final String apiKey;

  ServerApiKey(this.apiKey);

  @override
  String toString() => apiKey;
}

Future<ServerApiKey> authenticate(String username, String password,
    [IOClient? client]) async {
  client ??= IOClient(HttpClient()..findProxy = null);

  final response = await client.post(
    Uri.parse("https://api-icons.icons8.com/siteApi/auth/login"),
    body: {"email": username, "password": password},
    encoding: Encoding.getByName("UTF-8"),
  );

  if (response.statusCode != 200) {
    throw "Invalid response ${response.statusCode}: ${response.body}";
  }
  final body = json.decode(response.body);
  final apiKey = body["publicApiKey"] as String;

  return ServerApiKey(apiKey);
}

enum IconType { outline, solid }
